import type { aws_ec2, aws_ecs, Duration } from 'aws-cdk-lib';
/**
 * ecs.FargateServiceProps without taskDefinition and desiredCount
 *
 * @internal true
 */
export interface NarrowedFargateServiceProps {
    /**
     * The subnets to associate with the service.
     * @default - Public subnets if `assignPublicIp` is set, otherwise the first available one of Private, Isolated, Public, in that order.
     * @stability stable
     */
    readonly vpcSubnets?: aws_ec2.SubnetSelection;
    /**
     * The security groups to associate with the service.
     * If you do not specify a security group, a new security group is created.
     * @default - A new security group is created.
     * @stability stable
     */
    readonly securityGroups?: Array<aws_ec2.ISecurityGroup>;
    /**
     * The platform version on which to run your service.
     * If one is not specified, the LATEST platform version is used by default. For more information, see
     * [AWS Fargate Platform Versions](https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html)
     * in the Amazon Elastic Container Service Developer Guide.
     * @default Latest
     * @stability stable
     */
    readonly platformVersion?: aws_ecs.FargatePlatformVersion;
    /**
     * Whether to use Availability Zone rebalancing for the service.
     * If enabled, `maxHealthyPercent` must be greater than 100, and the service must not be a target
     * of a Classic Load Balancer.
     * @default AvailabilityZoneRebalancing.DISABLED
     * @stability stable
     */
    readonly availabilityZoneRebalancing?: aws_ecs.AvailabilityZoneRebalancing;
    /**
     * Specifies whether the task's elastic network interface receives a public IP address.
     * If true, each task will receive a public IP address.
     * @default false
     * @stability stable
     */
    readonly assignPublicIp?: boolean;
    /**
     * Configuration details for a volume used by the service.
     * This allows you to specify
     * details about the EBS volume that can be attched to ECS tasks.
     * @default - undefined
     * @stability stable
     */
    readonly volumeConfigurations?: Array<aws_ecs.ServiceManagedVolume>;
    /**
     * Revision number for the task definition or `latest` to use the latest active task revision.
     * @default - Uses the revision of the passed task definition deployed by CloudFormation
     * @stability stable
     */
    readonly taskDefinitionRevision?: aws_ecs.TaskDefinitionRevision;
    /**
     * The name of the service.
     * @default - CloudFormation-generated name.
     * @stability stable
     */
    readonly serviceName?: string;
    /**
     * Configuration for Service Connect.
     * @default No ports are advertised via Service Connect on this service, and the service
  cannot make requests to other services via Service Connect.
     * @stability stable
     */
    readonly serviceConnectConfiguration?: aws_ecs.ServiceConnectProps;
    /**
     * Specifies whether to propagate the tags from the task definition or the service to the tasks in the service.
     * Valid values are: PropagatedTagSource.SERVICE, PropagatedTagSource.TASK_DEFINITION or PropagatedTagSource.NONE
     * @default PropagatedTagSource.NONE
     * @stability stable
     */
    readonly propagateTags?: aws_ecs.PropagatedTagSource;
    /**
     * The minimum number of tasks, specified as a percentage of the Amazon ECS service's DesiredCount value, that must continue to run and remain healthy during a deployment.
     * @default - 0 if daemon, otherwise 50
     * @stability stable
     */
    readonly minHealthyPercent?: number;
    /**
     * The maximum number of tasks, specified as a percentage of the Amazon ECS service's DesiredCount value, that can run in a service during a deployment.
     * @default - 100 if daemon, otherwise 200
     * @stability stable
     */
    readonly maxHealthyPercent?: number;
    /**
     * The period of time, in seconds, that the Amazon ECS service scheduler ignores unhealthy Elastic Load Balancing target health checks after a task has first started.
     * @default - defaults to 60 seconds if at least one load balancer is in-use and it is not already set
     * @stability stable
     */
    readonly healthCheckGracePeriod?: Duration;
    /**
     * Whether to enable the ability to execute into a container.
     * @default - undefined
     * @stability stable
     */
    readonly enableExecuteCommand?: boolean;
    /**
     * Specifies whether to enable Amazon ECS managed tags for the tasks within the service.
     * For more information, see
     * [Tagging Your Amazon ECS Resources](https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html)
     * @default false
     * @stability stable
     */
    readonly enableECSManagedTags?: boolean;
    /**
     * Specifies which deployment controller to use for the service.
     * For more information, see
     * [Amazon ECS Deployment Types](https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html)
     * @default - Rolling update (ECS)
     * @stability stable
     */
    readonly deploymentController?: aws_ecs.DeploymentController;
    /**
     * The alarm(s) to monitor during deployment, and behavior to apply if at least one enters a state of alarm during the deployment or bake time.
     * @default - No alarms will be monitored during deployment.
     * @stability stable
     */
    readonly deploymentAlarms?: aws_ecs.DeploymentAlarmConfig;
    /**
     * The options for configuring an Amazon ECS service to use service discovery.
     * @default - AWS Cloud Map service discovery is not enabled.
     * @stability stable
     */
    readonly cloudMapOptions?: aws_ecs.CloudMapOptions;
    /**
     * Whether to enable the deployment circuit breaker.
     * If this property is defined, circuit breaker will be implicitly
     * enabled.
     * @default - disabled
     * @stability stable
     */
    readonly circuitBreaker?: aws_ecs.DeploymentCircuitBreaker;
    /**
     * A list of Capacity Provider strategies used to place a service.
     * @default - undefined
     * @stability stable
     */
    readonly capacityProviderStrategies?: Array<aws_ecs.CapacityProviderStrategy>;
    /**
     * The name of the cluster that hosts the service.
     * @stability stable
     */
    readonly cluster: aws_ecs.ICluster;
}
