import { aws_ecs as ecs, aws_efs as efs, aws_events as events, aws_lambda as lambda, aws_logs as logs } from 'aws-cdk-lib';
import { Construct } from 'constructs';
/**
 * A filter entry for a log.
 * @see https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/CloudWatch-Agent-Configuration-File-Details.html
 */
export interface LogFilter {
    /**
     * The type of filter. Either 'include' or 'exclude'.
     */
    readonly type: 'include' | 'exclude';
    /**
     * The filter pattern. This is a regular expression that matches the log lines to include or exclude.
     */
    readonly expression: string;
}
/**
 * A mapping of a log file to a CloudWatch log group.
 */
export interface LogMapping {
    /**
     * The path to the log file on the EFS access point.
     * This should be a relative path from the root of the access point.
     * It must not start with a '/'.
     *
     * @example 'my-log-file.log' if the main service mounts the logs access point at '/mnt/logs' and the file is at '/mnt/logs/my-log-file.log'. 'logs/my-log-file.log' if the main service mounts the logs access point at '/opt/app' and the file is at '/opt/app/logs/my-log-file.log/'.
     */
    readonly filePath: string;
    /**
     * The props for a log group to create.
     * Do not pass props for an existing log group here,
     * or a duplicate will be created and you will get
     * a deployment failure.
     *
     * If both `createLogGroup` and `logGroup` are absent,
     * a new log group will be created with a name
     * derived from last part of the file path
     * and default props.
     *
     * If both `createLogGroup` and `logGroup` are provided,
     * the `logGroup` will be used, and the `createLogGroup` will be ignored.
     *
     * @default - None
     */
    readonly createLogGroup?: logs.LogGroupProps;
    /**
     * The log group to use.
     *
     * This should be an existing log group.
     *
     * If both `createLogGroup` and `logGroup` are provided,
     * the `logGroup` will be used, and the `createLogGroup` will be ignored.
     *
     * @default - None
     */
    readonly logGroup?: logs.ILogGroup;
    /**
     * The format of the timestamp in the log file.
     * This is not quite the same as a strftime format string.
     * If this is not provided, the agent will forward all messages
     * with a timestamp matching the time it sees the message.
     *
     * @default - None
     */
    readonly timestampFormat?: string;
    /**
     * Whether to use UTC or local time for messages added to the log group.
     * Either 'UTC' or 'Local'. It does not allow arbitrary timezones to be set.
     * This is only used if `timestampFormat` is provided.
     * If this is not provided, the agent will use local time.
     *
     * @default - None
     */
    readonly timezone?: 'Local' | 'UTC';
    /**
     * The pattern to use for recognizing multi-line log messages.
     * This is a regular expression that matches the start of a new log message.
     * If this is not provided, the agent will treat each line that begins with
     * a non-whitespace character as starting a new log message.
     * If you include this field, you can specify `{timestamp_format}` to use
     * the same regular expression as your timestamp format. Otherwise, you can
     * specify a different regular expression for CloudWatch Logs to use to determine
     * the start lines of multi-line entries.
     *
     * @default - None
     */
    readonly multilinePattern?: string;
    /**
   * A list of filters to apply to the log file.
   * Each filter is a regular expression that matches the log lines to include or exclude.
   * If you include this field, the agent processes each log message with all of the
   * filters that you specify, and only the log events that pass all of the filters
   * are published to CloudWatch Logs. The log entries that don’t pass all of the
   * filters will still remain in the host's log file, but will not be sent to CloudWatch Logs.
   * If you omit this field, all logs in the log file are published to CloudWatch Logs.
   */
    readonly filters?: LogFilter[];
}
/**
 * Narrowed version of `FargateServiceProps` that omits the `taskDefinition` and `desiredCount` properties.
 * We don't need or want these properties, so we remove them from the type.
 *
 * This is generated by projen. See the .projenrc.ts and the NarrowedFargateServiceProps.ts files
 * for details.
 */
import { NarrowedFargateServiceProps } from './NarrowedFargateServiceProps.generated';
/**
 * Constructor properties for the FargateLogCollectorService.
 * This uses the private NarrowedFargateServiceProps interface in this project as a base.
 */
export interface FargateLogCollectorServiceProps extends NarrowedFargateServiceProps {
    /**
     * A list of log mappings.
     * This is used to create the CloudWatch agent configuration.
     * It is also used to create log groups if that is requested.
     * At least one log mapping must be provided.
     */
    readonly logMappings: LogMapping[];
    /**
     * The EFS access point where the logs are read from.
     */
    readonly efsLogsAccessPoint: efs.IAccessPoint;
    /**
     * The EFS access point where the agent state is stored.
     * This allows the agent to be restarted without losing its place in the logs.
     * Otherwise, the agent would forward every log file from the start
     * each time it is restarted.
     *
     * May be on a different EFS file system than `efsLogsAccessPoint` if desired.
     */
    readonly efsStateAccessPoint: efs.IAccessPoint;
    /**
     * The amount of memory (in MB) to allocate to the agent.
     * This is passed to the Fargate task definition.
     * You might need to increase this if you have a lot of logs to process.
     * Only some combinations of memory and CPU are valid.
     * @see https://docs.aws.amazon.com/cdk/api/v2/docs/aws-cdk-lib.aws_ecs.TaskDefinition.html#cpu
     *
     * @default 512
     */
    readonly agentMemory?: number;
    /**
     * The amount of CPU units to allocate to the agent.
     * 1024 CPU units = 1 vCPU.
     * This is passed to the Fargate task definition.
     * You might need to increase this if you have a lot of logs to process.
     * Only some combinations of memory and CPU are valid.
     * @see https://docs.aws.amazon.com/cdk/api/v2/docs/aws-cdk-lib.aws_ecs.TaskDefinition.html#memorymib
     *
     * @default 256
     */
    readonly agentCpu?: number;
    /**
     * The logging configuration for the container.
     *
     * @default ecs.LogDrivers.awsLogs({
                  logGroup: new logs.LogGroup(scope, 'ContainerLogGroup', {
                    logGroupName: `/${props.cluster.clusterName}/${props.serviceName || 'log-collector'}/ecs/tasks`,
                    retention: logs.RetentionDays.TWO_YEARS,
                    removalPolicy: RemovalPolicy.RETAIN_ON_UPDATE_OR_DELETE,
                  }),
                  streamPrefix: 'log-collector-task-logs',
                  mode: ecs.AwsLogDriverMode.NON_BLOCKING,
                 }),
     */
    readonly containerLogging?: ecs.LogDriver;
    /**
     * The log Group to use for the restart function.
     *
     * @default new logs.LogGroup(this, 'RestartServiceFunctionLogGroup', {
                  logGroupName: `/aws/lambda/${props.cluster.clusterName}/${props.serviceName || 'log-collector'}/restart-service`,
                  retention: logs.RetentionDays.TWO_YEARS,
                  removalPolicy: RemovalPolicy.RETAIN_ON_UPDATE_OR_DELETE,
                }),
     */
    readonly restartFunctionLogGroup?: logs.ILogGroup;
}
/**
 * A Fargate service that collects logs from a list of specified files on an EFS access point
 * and sends them to CloudWatch.
 * There is a single container in the task definition that runs the CloudWatch agent.
 *
 * There is a Lambda function included that restarts the agent service daily at 00:00 UTC.
 * This is to ensure that a new log stream is created every day for each log, named for the date,
 * with format `YYYY-MM-DD`. This is to make it easy to find the right log stream and
 * prevent the log streams from getting too large.
 *
 * There are two defaults changed from the parent class:
 *
 * - Deployment circuit breaker is enabled, with rollback, by default.
 * - minimum healthy percent is set to 0 by default, so we don't get a warning for having one task.
 */
export declare class FargateLogCollectorService extends ecs.FargateService {
    /**
     * Uniquely identifies this class.
     */
    static readonly PROPERTY_INJECTION_ID: string;
    /**
     * The EFS access point where the logs are read from.
     */
    readonly efsLogsAccessPoint: efs.IAccessPoint;
    /**
     * The EFS access point where the agent state is stored.
     * This allows the agent to be restarted without losing its place in the logs.
     * Otherwise, the agent would forward the whole logs each time it is restarted.
     */
    readonly efsStateAccessPoint: efs.IAccessPoint;
    /**
     * The list of log mappings, as processed from the input.
     * This can be used to see how the input log mappings were understood.
     * It is not the same as the log mappings passed in to the constructor.
     */
    readonly logMappings: LogMapping[];
    /**
     * The Lambda function that restarts the agent service.
     */
    readonly restartFunction: lambda.Function;
    /**
     * The CloudWatch event rule that triggers the Lambda function to restart the agent service.
     * It is set to run daily at 00:00 UTC. The purpose of this is to ensure that a new log stream
     * is created every day, named for the date.
     */
    readonly restartScheduleRule: events.Rule;
    /**
     * The amount of memory (in MB) allocated to the agent.
     * This was passed to the Fargate task definition.
     */
    readonly agentMemory: number;
    /**
     * The amount of CPU units allocated to the agent.
     * 1024 CPU units = 1 vCPU.
     * This was passed to the Fargate task definition.
     */
    readonly agentCpu: number;
    /**
     * The constructor for the FargateLogCollectorService.
     * This creates the IAM task and execution roles for the task, the task definition,
     * the CloudWatch agent container, the configuration for the agent, and the Fargate service.
     * It also creates a Lambda function that restarts the agent service.
     * @param scope The scope in which to create this Construct. Normally this is a stack.
     * @param id The Construct ID of the service.
     * @param props The properties for the service, as defined in the `FargateLogCollectorServiceProps` interface.
     */
    constructor(scope: Construct, id: string, props: FargateLogCollectorServiceProps);
}
